﻿namespace ConsumingBeacon
{
  using PublishingBeacon;
  using System;
  using Windows.Devices.Bluetooth.Advertisement;
  using Windows.Storage.Streams;
  using Windows.UI.Xaml;
  using Windows.UI.Xaml.Controls;

  public sealed partial class MainPage : Page
  {
    public MainPage()
    {
      this.InitializeComponent();
      this.Loaded += OnLoaded;
    }
    void OnLoaded(object sender, RoutedEventArgs e)
    {
      this.StartTimer();

      this.speechQueue = new SpeechQueue(this.mediaElement);

      this.StartWatchingForAdvertisements();
    }
    void StartWatchingForAdvertisements()
    {
      this.watcher = new BluetoothLEAdvertisementWatcher();

      BluetoothLEManufacturerData manufacturerData = 
        new BluetoothLEManufacturerData()
        {
          CompanyId = 0x010E
        };          

      this.watcher.AdvertisementFilter.Advertisement.ManufacturerData.Add(
        manufacturerData);

      watcher.SignalStrengthFilter.SamplingInterval = TimeSpan.FromMilliseconds(500);

      this.watcher.Received += OnAdvertisementReceived;

      watcher.Start();
    }
    void OnAdvertisementReceived(BluetoothLEAdvertisementWatcher sender,
      BluetoothLEAdvertisementReceivedEventArgs args)
    {
      foreach (var item in args.Advertisement.GetManufacturerDataByCompanyId(
        0x010E))
      {
        using (var dataReader = DataReader.FromBuffer(item.Data))
        {
          var length = dataReader.ReadInt32();

          this.currentRegistration = dataReader.ReadString((uint)length);
          this.currentSignalStrength = args.RawSignalStrengthInDBm;
        }
      }
    }
    void StartTimer()
    {
      this.timer = new DispatcherTimer();
      this.timer.Interval = TimeSpan.FromSeconds(2);
      this.timer.Tick += OnTimerTick;
      this.timer.Start();
    }
    void OnTimerTick(object sender, object e)
    {
      if (!string.IsNullOrEmpty(this.currentRegistration))
      {
        if (string.IsNullOrEmpty(this.previousRegistration))
        {
          this.speechQueue.SayAsync(
            $"found new car {this.currentRegistration}");
        }
        else
        {
          string speech = this.currentSignalStrength > this.previousSignalStrength ?
            "warmer" : "colder";

          // should really do some averaging and so on for these signal strengths as
          // they jump around a lot - not really a very reliable indicator of
          // distance the way I'm using it.
          if (Math.Abs(this.currentSignalStrength - this.previousSignalStrength) > SIGNAL_DELTA)
          {
            this.speechQueue.SayAsync(speech);
            this.previousSignalStrength = this.currentSignalStrength;
          }
        }
        this.previousRegistration = this.currentRegistration;
        this.txtCarReg.Text = this.currentRegistration;
        this.txtSignalStrength.Text = $"Signal Strength {this.currentSignalStrength}";
      }
      else if (!string.IsNullOrEmpty(this.previousRegistration))
      {
        this.speechQueue.SayAsync(
           $"lost car {this.previousRegistration}");

        this.previousRegistration = null;
        this.previousSignalStrength = 0;
        this.txtCarReg.Text = "NO REG";
        this.txtSignalStrength.Text = "Signal Strength 0";
      }
      this.currentRegistration = null;
      this.currentSignalStrength = 0;
    }
    string currentRegistration;
    string previousRegistration;
    int currentSignalStrength;
    int previousSignalStrength;
    DispatcherTimer timer;
    BluetoothLEAdvertisementWatcher watcher;
    SpeechQueue speechQueue;
    static readonly int SIGNAL_DELTA = 20;
  }
}