﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Media.SpeechSynthesis;
using Windows.UI.Xaml.Controls;

namespace PublishingBeacon
{
  class SpeechQueue
  {
    public SpeechQueue(MediaElement mediaElement)
    {
      this.synthesizer = new SpeechSynthesizer();
      this.mediaElement = mediaElement;
      this.mediaElement.MediaEnded += OnMediaEnded;
      this.outstandingSpeech = new Queue<string>();
    }

    void OnMediaEnded(object sender, Windows.UI.Xaml.RoutedEventArgs e)
    {
      this.speechStream.Dispose();
      this.taskSource.SetResult(true);            
    }
    public async Task SayAsync(string speechText)
    {
      this.outstandingSpeech.Enqueue(speechText);
      await this.DrainQueueAsync();
    }
    async Task DrainQueueAsync()
    {
      if (!this.draining)
      {
        this.draining = true;

        while (this.outstandingSpeech.Count() > 0)
        {
          string speech = this.outstandingSpeech.Dequeue();
  
          this.taskSource = new TaskCompletionSource<bool>();

          this.speechStream = await this.synthesizer.SynthesizeTextToStreamAsync(
            speech);

          this.mediaElement.SetSource(this.speechStream, string.Empty);

          this.mediaElement.Play();

          await this.taskSource.Task;

          this.taskSource = null;
        }
        this.draining = false;
      }
    }
    SpeechSynthesizer synthesizer;
    SpeechSynthesisStream speechStream;
    MediaElement mediaElement;
    TaskCompletionSource<bool> taskSource;
    bool draining;
    Queue<string> outstandingSpeech;
  }
}
